@@ -312,7 +312,7 @@ nimbleMCMC <- function(code,
         Rmodel <- if(is.Rmodel(model)) model else model$Rmodel
         if(!is.Rmodel(Rmodel)) stop('something went wrong')
     }
-    conf <- configureMCMC(Rmodel, monitors = monitors, thin = thin, enableWAIC = WAIC)
+    conf <- configureMCMC(Rmodel, monitors = monitors, thin = thin, enableWAIC = WAIC, print = FALSE)
     Rmcmc <- buildMCMC(conf)
     compiledList <- compileNimble(Rmodel, Rmcmc)    ## only one compileNimble() call
     Cmcmc <- compiledList$Rmcmc
