@@ -1121,13 +1121,19 @@ declare <- function(name, def){
 #'
 #' @param x vector of values
 #'
-#' @aliases is.nan.vec
+#' @aliases any_nan
 #' @author NIMBLE Development Team
 #' @export
+any_na <- function(x) any(is.na(x))
+
+## Retained for backwards compatibility
 is.na.vec <- function(x) any(is.na(x))
 
-#' @rdname is.na.vec
+#' @rdname any_na
 #' @export
+any_nan <- function(x) any(is.nan(x))
+
+## Retained for backwards compatibility
 is.nan.vec <- function(x) any(is.nan(x))
 
 #' @export
