@@ -722,34 +722,6 @@ symbolInternalType <-
                     })
                 )
 
-## Object for Tensorflow runner.
-symbolTensorflowRunner <-
-    setRefClass(Class = "symbolTensorflowRunner",
-                contains = "symbolBase",
-                fields = list(constructor = "ANY"),
-                methods = list(
-                    initialize = function(...) callSuper(...),
-                    show = function() writeLines(paste('symbolTensorflowRunner ', name)),
-                    genCppVar = function(functionArg = FALSE) {
-                        cppVarFull(name = name, baseType = "NimTf_Runner", static = TRUE, ref = TRUE, constructor = constructor)
-                    }
-                )
-    )
-
-## Object for wrapping a Tensorflow runner in a CppAD op.
-symbolTensorflowOp <-
-    setRefClass(Class = "symbolTensorflowOp",
-                contains = "symbolBase",
-                fields = list(constructor = "ANY"),
-                methods = list(
-                    initialize = function(...) callSuper(...),
-                    show = function() writeLines(paste('symbolTensorflowOp ', name)),
-                    genCppVar = function(functionArg = FALSE) {
-                        cppVarFull(name = name, baseType = "NimTf_Op", static = TRUE, ref = TRUE, constructor = constructor)
-                    }
-                )
-    )
-
 ## nDim is set to length(size) unless provided, which is how scalar (nDim = 0) must be set
 symbolDouble <- function(name, size = numeric(), nDim = length(size)) {
     if(is.logical(size)) size <- as.numeric(size)
