@@ -546,6 +546,155 @@ test_that("ddexp usage", {
     set.seed(1)
     cm$simulate('mu')
     expect_identical(smp[1], cm$mu, "incorrect compiled simulate for ddexp")
+
+    ## check use of rate
+    set.seed(1)
+    x <- rdexp(1, 0.5, scale = 0.25)
+    d1 <- ddexp(x, 0.5, 0.25)
+    d2 <- ddexp(x, 0.5, rate = 4)
+    expect_identical(d1, d2)
+
+    ## nimbleFunction
+    f <- nimbleFunction(
+        run = function(x = double(), location = double(), par2 = double()) {
+            returnType(double(1))
+            out1 <- ddexp(x, location, par2)
+            out2 <- ddexp(x, location, rate = 1/par2)
+            return(c(out1, out2))
+        })
+    cf <- compileNimble(f)
+    out <- cf(x, 0.5, 0.25)
+    expect_identical(out, c(d1, d2))
+
+    ## in a model
+    code <- nimbleCode({
+        x1 ~ ddexp(mu, scale = scale)
+        x2 ~ ddexp(mu, rate)
+    })
+    m <- nimbleModel(code, data = list(x1 = x, x2 = x), inits = list(mu = 0.5, scale = 0.25, rate = 4))
+    cm <- compileNimble(m)
+    out1 <- cm$calculate('x1')
+    out2 <- cm$calculate('x2')
+    expect_identical(exp(c(out1, out2)), c(d1, d2))
+
+})
+
+
+test_that("recycling behavior from R and within nimbleFunctions for non-R-native distributions", {
+
+    ## dt_nonstandard
+    set.seed(1)
+    param <- runif(3)
+    x <- rt_nonstandard(6, 3, param, 3.5)
+    expect_equal(length(x), 6)
+    param <- rep(param, 2)
+    d <- dt_nonstandard(x[1:3], 3, param, 3.5)
+    expect_identical(d[1:3], d[4:6])
+    p <- pt_nonstandard(x[1:3], 3, param, 3.5)
+    q <- qt_nonstandard(p, 3, param, 3.5)
+    expect_equal(rep(x[1:3], 2), q)
+    expect_identical(p[1:3], p[4:6])
+    expect_identical(q[1:3], q[4:6])
+
+    ## Use of recycling
+    f <- nimbleFunction(
+        run = function(x = double(1), theta = double(1)) {
+            dd <- dt_nonstandard(x, 3, theta, 3.5)
+            pp <- pt_nonstandard(x, 3, theta, 3.5)
+            qq <- qt_nonstandard(pp, 3, theta, 3.5)
+            returnType(double(1))
+            return(c(dd, pp, qq))
+        })
+    cf <- compileNimble(f)
+    out <- cf(x[1:3], param) 
+    expect_identical(out, c(d, p, q), info = 'dt_nonstandard nf')
+
+    ## dexp_nimble
+    set.seed(1)
+    param <- runif(3)
+    x <- rexp_nimble(6, param)
+    expect_equal(length(x), 6)
+    param <- rep(param, 2)
+    d <- dexp_nimble(x[1:3], param)
+    expect_identical(d[1:3], d[4:6])
+    p <- pexp_nimble(x[1:3], param)
+    q <- qexp_nimble(p, param)
+    expect_equal(rep(x[1:3], 2), q)
+    expect_identical(p[1:3], p[4:6])
+    expect_identical(q[1:3], q[4:6])
+
+    f <- nimbleFunction(
+        run = function(x = double(1), theta = double(1)) {
+            d <- dexp_nimble(x, theta)
+            p <- pexp_nimble(x, theta)
+            q <- qexp_nimble(p, theta)
+            returnType(double(1))
+            return(c(d, p, q))
+        })
+    cf <- compileNimble(f)
+    out <- cf(x[1:3], param) 
+    expect_identical(out, c(d, p, q), info = 'dexp_nimble nf')
+
+    ## ddexp
+    set.seed(1)
+    param <- runif(3)
+    x <- rdexp(6, 0.5, param)
+    expect_equal(length(x), 6)
+    param <- rep(param, 2)
+    d <- ddexp(x[1:3], 0.5, param)
+    expect_identical(d[1:3], d[4:6])
+    p <- pdexp(x[1:3], 0.5, param)
+    q <- qdexp(p, 0.5, param)
+    expect_equal(rep(x[1:3], 2), q)
+    expect_identical(p[1:3], p[4:6])
+    expect_identical(q[1:3], q[4:6])
+
+    f <- nimbleFunction(
+        run = function(x = double(1), theta = double(1)) {
+            d <- ddexp(x, 0.5, theta)
+            p <- pdexp(x, 0.5, theta)
+            q <- qdexp(p, 0.5, theta)
+            returnType(double(1))
+            return(c(d, p, q))
+        })
+    cf <- compileNimble(f)
+    out <- cf(x[1:3], param) 
+    expect_identical(out, c(d, p, q), info = 'ddexp nf')
+
+    ## dsqrt_invgamma (no p or q functions; not available in nimbleFunction)
+    set.seed(1)
+    param <- runif(3)
+    x <- rsqrtinvgamma(6, 0.5, param)
+    expect_equal(length(x), 6)
+    param <- rep(param, 2)
+    d <- dsqrtinvgamma(x[1:3], 0.5, param)
+    expect_identical(d[1:3], d[4:6])
+    
+    ## dinvgamma
+    set.seed(1)
+    param <- runif(3)
+    x <- rinvgamma(6, 0.5, param)
+    expect_equal(length(x), 6)
+    param <- rep(param, 2)
+    d <- dinvgamma(x[1:3], 0.5, param)
+    expect_identical(d[1:3], d[4:6])
+    p <- pinvgamma(x[1:3], 0.5, param)
+    q <- qinvgamma(p, 0.5, param)
+    expect_equal(rep(x[1:3], 2), q)
+    expect_identical(p[1:3], p[4:6])
+    expect_identical(q[1:3], q[4:6])
+
+    f <- nimbleFunction(
+        run = function(x = double(1), theta = double(1)) {
+            d <- dinvgamma(x, 0.5, theta)
+            p <- pinvgamma(x, 0.5, theta)
+            q <- qinvgamma(p, 0.5, theta)
+            returnType(double(1))
+            return(c(d, p, q))
+        })
+    cf <- compileNimble(f)
+    out <- cf(x[1:3], param) 
+    expect_identical(out, c(d, p, q), info = 'dinvgamma nf')
 })
     
    
